<?xml version="1.0" encoding="UTF-8"?>
<xsl:transform version="1.0" 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:set="http://exslt.org/sets"
    xmlns:common="http://exslt.org/common"
    xmlns:monitoring="http://schemas.qoscosgrid.org/comp/2011/04/monitoring"
    xmlns:factory="http://schemas.qoscosgrid.org/comp/2011/04/factory"
    xmlns:bes-factory="http://schemas.ggf.org/bes/2006/08/bes-factory">

	<xsl:strip-space elements="*"/>
	<xsl:output method="xml" version="1.0" encoding="UTF-8" omit-xml-declaration="yes" indent="yes"/>
	<!-- =====================================================================================================================-->

	<xsl:variable name="RESOURCES" select="/bes-factory:FactoryResourceAttributesDocumentType/bes-factory:ContainedResource"/>
	<xsl:variable name="APPLICATIONS" select="/bes-factory:FactoryResourceAttributesDocumentType/factory:Applications/factory:Application"/>
	<xsl:variable name="MODULES" select="/bes-factory:FactoryResourceAttributesDocumentType/factory:Modules/factory:Module"/>
	<xsl:variable name="USERS" select="/bes-factory:FactoryResourceAttributesDocumentType/factory:PlGridInfo/factory:Users/factory:User"/>
	<xsl:variable name="QUEUES" select="/bes-factory:FactoryResourceAttributesDocumentType/monitoring:ClusterMonitoring/monitoring:Queues/monitoring:Queue"/>
	<xsl:variable name="JOBS" select="/bes-factory:FactoryResourceAttributesDocumentType/monitoring:ClusterMonitoring/monitoring:Jobs/monitoring:Job"/>
	<xsl:variable name="RESERVATIONS" select="/bes-factory:FactoryResourceAttributesDocumentType/monitoring:ClusterMonitoring/monitoring:Reservations/monitoring:Reservation"/>
	<xsl:variable name="VOS" select="/bes-factory:FactoryResourceAttributesDocument/factory:AuthorizedVO/factory:VO"/>

	<!-- =====================================================================================================================-->
	<xsl:key name="RESOURCE_STATE" match="/bes-factory:FactoryResourceAttributesDocumentType/bes-factory:ContainedResource" use="monitoring:State"/>
	<xsl:key name="GRANTS" match="/bes-factory:FactoryResourceAttributesDocumentType/factory:PlGridInfo/factory:Groups/factory:Group/factory:Grant" use="../@name"/>
	<!-- =====================================================================================================================-->
	<xsl:template match="/">
	    <xsl:element name="resourceDescription">
	        <xsl:attribute name="timestamp">
		    <xsl:value-of select="//factory:Timestamp"/>
		</xsl:attribute>

		<xsl:call-template name="SUMMARY"/>
		<xsl:call-template name="NODES"/> 
		<xsl:call-template name="APPLICATIONS"/>
 		<xsl:call-template name="MODULES"/> 
		<xsl:call-template name="USERS"/>
		<xsl:call-template name="QUEUES"/>
		<xsl:call-template name="VOS"/>
	    </xsl:element>
	</xsl:template>

	<!-- =====================================================================================================================-->
	<xsl:template name="SUMMARY">
	    <xsl:element name="summary">
		<xsl:element name="nodes">
		    <xsl:attribute name="count">
			<xsl:value-of select="count($RESOURCES)"/>
		    </xsl:attribute>
		<xsl:for-each select="set:distinct($RESOURCES/monitoring:State)">
		    <xsl:element name="state">
			<xsl:attribute name="name">
			    <xsl:value-of select="."/>
			</xsl:attribute>
			<xsl:value-of select="count(key('RESOURCE_STATE',.))"/>
		    </xsl:element>
		</xsl:for-each>
		</xsl:element>
		<xsl:element name="applications">
		    <xsl:attribute name="count">
			<xsl:value-of select="count($APPLICATIONS)"/>
		    </xsl:attribute>
		</xsl:element>
		<xsl:element name="modules">
		    <xsl:attribute name="count">
			<xsl:value-of select="count($MODULES)"/>
		    </xsl:attribute>
		</xsl:element>
		<xsl:element name="users">
		    <xsl:attribute name="count">
			<xsl:value-of select="count($USERS)"/>
		    </xsl:attribute>
		</xsl:element>
		<xsl:element name="queues">
		    <xsl:attribute name="count">
			<xsl:value-of select="count($QUEUES)"/>
		    </xsl:attribute>
		</xsl:element>
		<xsl:element name="jobs">
		    <xsl:attribute name="count">
			<xsl:value-of select="count($JOBS)"/>
		    </xsl:attribute>
		    <xsl:attribute name="running">
			<xsl:value-of select="sum($QUEUES/@runningJobs)"/>
		    </xsl:attribute>
		    <xsl:attribute name="waiting">
			<xsl:value-of select="sum($QUEUES/@waitingJobs)"/>
		    </xsl:attribute>
		</xsl:element>
		<xsl:element name="reservations">
		    <xsl:attribute name="count">
			<xsl:value-of select="count($RESERVATIONS)"/>
		    </xsl:attribute>
		</xsl:element>
		<xsl:element name="vos">
		    <xsl:attribute name="count">
			<xsl:value-of select="count($VOS)"/>
		    </xsl:attribute>
		</xsl:element>
	    </xsl:element>
	</xsl:template>

	<!-- =====================================================================================================================-->
	<xsl:template name="NODES">
	    <xsl:if test="$RESOURCES">
	        <xsl:element name="nodes">
			<xsl:apply-templates select="$RESOURCES"/>
		</xsl:element>
	    </xsl:if>
        </xsl:template>

	<xsl:template match="bes-factory:ContainedResource">
	    <xsl:element name="node">
		<xsl:attribute name="name">
		    <xsl:value-of select="bes-factory:ResourceName"/>
		</xsl:attribute>
		<xsl:if test="bes-factory:CPUCount">
		    <xsl:attribute name="cores">
			<xsl:value-of select="round(bes-factory:CPUCount)"/>
		    </xsl:attribute>
		</xsl:if>
		<xsl:if test="monitoring:UsedCPUCount">
		    <xsl:attribute name="used">
			<xsl:value-of select="monitoring:UsedCPUCount"/>
		    </xsl:attribute>
		</xsl:if>
		<xsl:if test="bes-factory:PhysicalMemory">
		    <xsl:attribute name="memory">
			<xsl:value-of select="bes-factory:PhysicalMemory"/>
		    </xsl:attribute>
		</xsl:if>
		<xsl:if test="monitoring:Attributes">
		    <xsl:attribute name="attributes">
			<xsl:value-of select="monitoring:Attributes"/>
		    </xsl:attribute>
		</xsl:if>
		<xsl:if test="monitoring:State">
		    <xsl:attribute name="state">
			<xsl:value-of select="monitoring:State"/>
		    </xsl:attribute>
		</xsl:if>
	    </xsl:element>
	</xsl:template>

	<!-- =====================================================================================================================-->
	<xsl:template name="APPLICATIONS">
	    <xsl:if test="$APPLICATIONS">
	        <xsl:element name="applications">
			<xsl:apply-templates select="$APPLICATIONS"/>
		</xsl:element>
	    </xsl:if>
        </xsl:template>

	<xsl:template match="factory:Application">
	    <xsl:element name="application">
		<xsl:attribute name="name">
		    <xsl:value-of select="@name"/>
		</xsl:attribute>
		<xsl:if test="@version">
		    <xsl:attribute name="version">
			<xsl:value-of select="@version"/>
		    </xsl:attribute>
		</xsl:if>
	    </xsl:element>
	</xsl:template>

	<!-- =====================================================================================================================-->
	<xsl:template name="MODULES">
	    <xsl:if test="$MODULES">
	        <xsl:element name="modules">
			<xsl:apply-templates select="$MODULES"/>
		</xsl:element>
	    </xsl:if>
        </xsl:template>

	<xsl:template match="factory:Module">
	    <xsl:element name="module">
		<xsl:attribute name="name">
		    <xsl:value-of select="@name"/>
		</xsl:attribute>
		<xsl:if test="@version">
		    <xsl:attribute name="version">
			<xsl:value-of select="@version"/>
		    </xsl:attribute>
		</xsl:if>
	    </xsl:element>
	</xsl:template>


	<!-- =====================================================================================================================-->
	<xsl:template name="USERS">
	    <xsl:if test="$USERS">
	        <xsl:element name="users">
			<xsl:apply-templates select="$USERS"/>
		</xsl:element>
	    </xsl:if>
        </xsl:template>

	<xsl:template match="factory:User">
	    <xsl:element name="user">
		<xsl:attribute name="name">
		    <xsl:value-of select="@name"/>
		</xsl:attribute>
		<xsl:apply-templates select="factory:DN"/>
		<xsl:variable name="USER_GRANTS">
		    <xsl:for-each select="factory:Group">
                	<xsl:copy-of select="key('GRANTS',.)"/>
            	    </xsl:for-each>    
		</xsl:variable>

		<xsl:if test="factory:PrivateGrant or factory:DefaultGrant or common:node-set($USER_GRANTS)/*">
		    <xsl:element name="grants">
			<xsl:if test="factory:PrivateGrant">
			    <xsl:attribute name="private">
				<xsl:value-of select="factory:PrivateGrant"/>
			    </xsl:attribute>
			</xsl:if>
			<xsl:if test="factory:DefaultGrant">
			    <xsl:attribute name="default">
				<xsl:value-of select="factory:DefaultGrant"/>
			    </xsl:attribute>
			</xsl:if>
                	<xsl:apply-templates select="common:node-set($USER_GRANTS)/*"/>
		    </xsl:element>
		</xsl:if>
	    </xsl:element>
	</xsl:template>

	<xsl:template match="factory:DN">
	    <xsl:element name="DN">
		<xsl:value-of select="."/>
	    </xsl:element>
	</xsl:template>

	<xsl:template match="factory:Grant">
	    <xsl:element name="grant">
		<xsl:value-of select="."/>
	    </xsl:element>
	</xsl:template>


	<!-- =====================================================================================================================-->
	<xsl:template name="QUEUES">
	    <xsl:if test="$QUEUES">
	        <xsl:element name="queues">
			<xsl:apply-templates select="$QUEUES"/>
		</xsl:element>
	    </xsl:if>
        </xsl:template>

	<xsl:template match="monitoring:Queue">
	    <xsl:element name="queue">
		<xsl:apply-templates select="@*[name()!='default']"/>
	    </xsl:element>
	</xsl:template>

	<!-- =====================================================================================================================-->
	<xsl:template name="VOS">
	    <xsl:if test="$VOS">
	        <xsl:element name="vos">
			<xsl:apply-templates select="$VOS"/>
		</xsl:element>
	    </xsl:if>
        </xsl:template>

	<xsl:template match="factory:VO">
	    <xsl:element name="vo">
		<xsl:attribute name="name">
		    <xsl:value-of select="."/>
		</xsl:attribute>
	    </xsl:element>
	</xsl:template>


	<!-- =====================================================================================================================-->
	<xsl:template match="*|@*|text()">
            <xsl:copy>
                <xsl:apply-templates select="*|@*|text()"/>
            </xsl:copy>
        </xsl:template>
</xsl:transform>
